/*--------------------------VC_Sequential_Switch_101.c  -----------------------------------
 * 
 * 
 * Hardware: ATMega 328p mit 8MHz internal Clock
 * Board: Rotating Gate 090-F and 090-B 
 * 
 * 
 * avrdude -p m328p -c avrisp2                      ; Console command
 * avrdude: Device signature = 0x1e950f             ; default output
 * avrdude: safemode: Fuses OK (E:07, H:D9, L:62)   ; default output
 * 
 * 
 * 
 * Fuse setting for 8MHz internal clock:
 * avrdude -p m328p -c avrisp2 -U lfuse:w:0xe2:m 
 * avrdude -p m328p -c avrisp2 -U hfuse:w:0xd9:m 
 * 
 * 
 * 
 * compile:
 * avr-gcc -Wall -Os -mmcu=atmega328p VC_Sequential_Switch_101.c -o VC_Sequential_Switch_101.elf
 * avr-objcopy -O ihex -j .text -j .data VC_Sequential_Switch_101.elf VC_Sequential_Switch_101.hex
 * avrdude -p m328p -c avrisp2 -U flash:w:VC_Sequential_Switch_101.hex
 * 
 * 
 * ADC0: Steps potentiometer
 * ADC1: Mode potentiometer
 * INT0: Rising edge
 * INT1: Falling edge / Currently not used here
 * PD7: Pulse in LED
 * PB0 - PB7: Switch outputs
 * 
 * 
 * 
 * 
 * 
 * 20240118 Start Harald
 * 
 * 
 * ---------------------------------------------------------------------------------------*/

#include <avr/io.h>
#include <avr/interrupt.h>


/* ------------ Variables --------------------------*/

volatile uint8_t max_steps;				// Holds maximum used steps derived from ADC0
volatile uint8_t steps_read;			// Holds result from ADC0 steps pot and innput
volatile uint8_t count_steps;			// Counts done steps
volatile uint8_t mode_read;				// Holds result from ADC1 mode pot and innput
volatile uint8_t mode;						// Holds mode derived from ADC1; 
                                  // mode 1 = step up
                                  // mode 2 = step down 


uint8_t step_up[] = 
    { 
0b00000000, 
0b00000001,
0b00000010,
0b00000100,
0b00001000,
0b00010000,
0b00100000,
0b01000000,
0b10000000
    };

		
uint8_t step_down[] = 
    { 
0b00000000, 
0b10000000,
0b01000000,
0b00100000,
0b00010000,
0b00001000,
0b00000100,
0b00000010,
0b00000001
    };		
		
uint8_t step_odd[] = 
    { 
0b00000000, 
0b01010101,
0b10101010,
0b01010101,
0b10101010,
0b01010101,
0b10101010,
0b01010101,
0b10101010
    };
		
uint8_t step_lr[] = 
    { 
0b00000000, 
0b00001111,
0b11110000,
0b00001111,
0b11110000,
0b00001111,
0b11110000,
0b00001111,
0b11110000
    };
		
/* Interrupt vectors */

//INT0 interrupt 
ISR(INT0_vect )																// Pulse in goes high
    
    { 
				if ((max_steps == 0) || (mode == 0))	// All outputs stay off
        {
					PORTD = 0b10000000;									// Pulse LED on. Signaling incoming pulse high
					PORTB = 0b00000000;									// All outputs off
        }
				else 
				{	
					switch (mode)
					{
						case 1 : 	PORTB = step_up[count_steps];			// Output array[count_steps]
											PORTD = 0b10000000;								// Pulse LED on
											break;
						case 2 :	PORTB = step_down[count_steps];		// Output array[count_steps]
											PORTD = 0b10000000;								// Pulse LED on
											break;
						case 3 :	PORTB = step_odd[count_steps];		// Output array[count_steps]
											PORTD = 0b10000000;								// Pulse LED on
											break;
						case 4 :	PORTB = step_lr[count_steps];			// Output array[count_steps]
											PORTD = 0b10000000;								// Pulse LED on
											break;
						case 5 : 	PORTB = step_up[count_steps];			// Output array[count_steps]
											PORTD = 0b10000000;								// Pulse LED on
											break;
						case 6 :	PORTB = step_down[count_steps];		// Output array[count_steps]
											PORTD = 0b10000000;								// Pulse LED on
											break;
						case 7 :	PORTB = step_odd[count_steps];		// Output array[count_steps]
											PORTD = 0b10000000;								// Pulse LED on
											break;
						case 8 :	PORTB = step_lr[count_steps];			// Output array[count_steps]
											PORTD = 0b10000000;								// Pulse LED on
											break;
					}
				}
				if (count_steps >= max_steps)   // Cycle through array until max_steps reached
					count_steps = 0;							// Start again on top
				if (max_steps > 0)
          count_steps++;								// increase step counter
    } 

    

ISR(INT1_vect)        	      // Clock in goes low
{
    PORTD = 0b00000000;		   	// Pulse LED off. Signaling incoming pulse low
}


/* ADC init */

void ADC_Init(void) {
 
  uint16_t result;
 
  ADMUX |= (1 << REFS0);					// reference voltage on AVCC    
	ADMUX |= (1 << ADLAR);					// left adjust result, return only 8bits
  // Bit ADFR ("free running") in ADCSRA is already set to zero on startup; single conversion
  ADCSRA |= (1<<ADPS2);    				// Prescaler 8MHz / 16 = 250kHz empf: 50 - 200 kHz
  ADCSRA |= (1<<ADEN);            // ADC activate
  ADCSRA |= (1<<ADSC);						// eine ADC-Wandlung 
  while (ADCSRA & (1<<ADSC) )			// Wait for end of conversion
	{         	
  }
  result = ADCW;									//ADCW must be read once
}


/* ADC single measure */
uint8_t ADC_Read( uint8_t channel )
{
  // Choose channnel
  ADMUX = (ADMUX & ~(0x1F)) | (channel & 0x1F);
  ADCSRA |= (1<<ADSC);            // single conversion
  while (ADCSRA & (1<<ADSC) ) 		// Wait for end of conversion
	{   
  }
  return (ADCH);           		    // ADC High byte auslesen und zurückgeben
}                             
  
      

/* ----------------------MAIN ---------------------------*/
int main (void)
{					
  DDRD &=~ (1 << PD3);         	  // set PD3 as input; INT0  
  DDRD &=~ (1 << PD2);            // set PD2 as input; INT1  
	DDRD |= (1 << PD7);							// set PD7 as output for Gate LED
  DDRB = 0b11111111;              // PORTB output
  EIMSK |= (1<<INT0);			        // enable INT0
  EIMSK |= (1<<INT1);			        // enable INT1
  EICRA |= (1<<ISC00)|(1<<ISC01); // INT0 - Rising edge on INT0 generates an interrupt request
  EICRA |= (1<<ISC11);            // INT1 - Falling edge of INT1 generates interrupt request
  PORTB = 0b00000000;             // All outputs to zero
  count_steps = 0;
	max_steps = 0;
  mode = 0;
	ADC_Init();
  sei();			                    // Interrupts activate
  while (1)                       // for ever
    {				
			steps_read = ADC_Read(0);		// Steps pot and input read	
			if (steps_read <= 2)
				max_steps = 0;
			if ((steps_read > 2) && (steps_read <= 32))
				max_steps = 1;
			if ((steps_read > 32) && (steps_read <= 62))
				max_steps = 2;
			if ((steps_read > 62) && (steps_read <= 92))
				max_steps = 3;
			if ((steps_read > 92) && (steps_read <= 122))
				max_steps = 4;
			if ((steps_read > 122) && (steps_read <= 152))
				max_steps = 5;
			if ((steps_read > 152) && (steps_read <= 182))
				max_steps = 6;
			if ((steps_read > 182) && (steps_read <= 212))
				max_steps = 7;
			if (steps_read > 212)
				max_steps = 8;
      
      mode_read = ADC_Read(1);		// Mode pot and input read	
			if (mode_read <= 2)
				mode = 0;
			if ((mode_read > 2) && (mode_read <= 32))
				mode = 1;
			if ((mode_read > 32) && (mode_read <= 62))
				mode = 2;
			if ((mode_read > 62) && (mode_read <= 92))
				mode = 3;
			if ((mode_read > 92) && (mode_read <= 122))
				mode = 4;
			if ((mode_read > 122) && (mode_read <= 152))
				mode = 5;
			if ((mode_read > 152) && (mode_read <= 182))
				mode = 6;
			if ((mode_read > 182) && (mode_read <= 212))
				mode = 7;
			if (mode_read > 212)
				mode = 8;
    }
}
